<?php

/**
 * @file
 * Kewego engine classes.
 */

class DatasourceEngineKewegoException extends DatasourceEngineException {

}

/**
 * Abstract Kewego datasource engine.
 */
abstract class DatasourceEngineKewegoAbstract extends DatasourceEngineAbstract {
  protected function getAppKey() {
    return variable_get('kewego_application_key', '');
  }
  
  protected $live;
  
  /**
   * @var KewegoApi
   */
  protected $kewego;
  
  public function initImport() {
    if (!parent::initImport()) return FALSE;
    
    if (module_exists('process_api')) {
      $this->live = process_api_live();
      process_api_live(FALSE);
    }
    
    $this->kewego = kewego_api();
 
    return TRUE;
  }
  
  public function completeImport() {
   if (module_exists('process_api')) {
     process_api_live($this->live);
   }
   parent::completeImport();
  }

  protected function unaccent($text) {
    static $search, $replace;
    if (!$search) {
      $search = $replace = array();
      // Get the HTML entities table into an array
      $trans = get_html_translation_table(HTML_ENTITIES);
      // Go through the entity mappings one-by-one
      foreach ($trans as $literal => $entity) {
        // Make sure we don't process any other characters
        // such as fractions, quotes etc:
        if (ord($literal) >= 192) {
          // Get the accented form of the letter
          $search[] = utf8_encode($literal);
          // Get e.g. 'E' from the string '&Eacute'
          $replace[] = $entity[1];
        }
      }
    }
    return str_replace($search, $replace, $text);
  }

}

/**
 * Article Protec datasource engine.
 */
class DatasourceEngineKewegoVideo extends DatasourceEngineKewegoAbstract {

  protected $xml;
  
  protected $items = array();
  
   public function initImport() {
     if (!parent::initImport()) return FALSE;
     if (empty($this->flags['sig'])) {
       if ($this->fetchItems()) {
         return TRUE;
       }
     }
     else {
       // import only specified video (Quick Import)
       if ($this->fetchVideo($this->flags['sig'])) {
         return TRUE;
       }
     }
     return FALSE;
   }
  
   protected function fetchItems() {
     $this->xml = $this->kewego->apiCall('search/getLatestVideos', array(
         'max_result_per_page' => $this->getFetchRange(),
         'starting' => $this->getFetchOffset(),
         'fullDescription' => $this->getFullDescription(),
         'format_thumb' => $this->getThumbnailSize(),
     ));
      
     if ($this->xml) {
       foreach ($this->xml->item as $xml_item) {
         $this->items[(string)$xml_item->sig] = $this->xml2object($xml_item);
       }
       return TRUE;
     }
     return FALSE;
   }
   
   protected function fetchVideo($sig) {
     $this->xml = $this->kewego->apiCall('video/getDetails', array(
         'sig' => $sig,
     ));
     if ($this->xml) {
       $this->xml->sig = $sig;
       $this->items[$sig] = $this->xml2object($this->xml);
       return TRUE;
     }
     return FALSE;
   }
   
   protected function xml2object($xml_item) {
     $item = (object) array();
     foreach (array('sig', 'title', 'description', 'author', 'rating', 'duration', 'views', 'thumbnail') as $field) {
       if (isset($xml_item->$field)) $item->$field = (string)$xml_item->$field;
     }
     foreach ($xml_item->categories->category as $xml_category) {
       $item->categories[] = (string) $xml_category;
     }
     
     // difference between video/getDetails and search/getLatestVideos
     if (isset($xml_item->keywords->keywords)) {
       foreach ($xml_item->keywords->keywords as $xml_keyword) {
         $item->keywords[] = (string) $xml_keyword;
       }
     }
     else {
       foreach ($xml_item->keywords->keyword as $xml_keyword) {
         $item->keywords[] = (string) $xml_keyword;
       }
     }
     
     foreach (array('pubdate', 'production_date', 'publication_date') as $field) {
       if (!isset($xml_item->$field)) continue;
       $value = (string) $xml_item->$field;
       // difference between video/getDetails and search/getLatestVideos
       if ($field == 'publication_date') $field = 'pubdate';
       if (preg_match('|^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2})\:([0-9]{2})\:([0-9]{2})$|', $value, $matches)) {
         if ($matches[1] == 0) $item->$field = null;
         else $item->$field = mktime($matches[4], $matches[5], $matches[6], $matches[2], $matches[3], $matches[1]);
       }
       else $item->$field = null;
     }
     
     // difference between video/getDetails and search/getLatestVideos
     if (!isset($item->thumbnail)) {
       $size = $this->getThumbnailSize();
       if (isset($xml_item->thumbnails->$size)) {
         $item->thumbnail = (string) reset($xml_item->thumbnails->$size->url);
       }
       else {
         $item->thumbnail = '';
       }
     }
     
     $item->sig_udid = array(
         'source' => 'datasource_kewego_videos',
         'id' => $item->sig,
         'branch' => 'main',
         'version' => $item->pubdate);
     
     $item->description = trim(strip_tags(html_entity_decode(strtr($item->description, array('&nbsp;' => ' ')), ENT_COMPAT, 'UTF-8')));

     $item->{'#provider'} = 'Kewego';
     
     return $item;
   }
   
   protected function getFetchRange() {
     return variable_get('datasource_kewego_fetch_range', 50);
   }
   
   protected function getThumbnailSize() {
     return variable_get('datasource_kewego_thumbnail_size', 'large');
   }
   
   protected function getFetchOffset() {
     return function_exists('drush_get_option') ? drush_get_option('offset', 0) : 0;
   }
   
   protected function getFullDescription() {
     return variable_get('datasource_kewego_full_description', 1);
   }
   
  protected function getRecord() {
    list($sig, $item) = each($this->items);
    if (!$item) return null;
    echo "Importing {$item->sig} / {$item->title}...\n";
    return $item;
  }

  protected function onErrorImportRecord($record, array $context) {
    
  }  
  
  protected function beforeSaveEntity($entity, array $context) {
    
  }

}
